@@ -50,6 +50,14 @@ static void print_help(void);
 static char *tilde_expansion(char *str);
 static char *nvstrcat(const char *str, ...);
 
+/*
+ * verbosity, controls output of errors, warnings and other
+ * information (used by msg.c).
+ */
+
+int __verbosity = VERBOSITY_DEFAULT;
+
+
 /*
  * print_version() - print version information
  */
@@ -110,6 +118,21 @@ static const NVGetoptOption __options[] = {
       "the X server, and exit.  This mode of operation is useful to place "
       "in your .xinitrc file, for example." },
 
+    { "no-config", 'n', 0, NULL,
+      "Do not load the configuration file.  This mode of operation is useful "
+      "if nvidia-settings has difficulties starting due to problems with "
+      "applying settings in the configuration file." },
+
+    { "rewrite-config-file", 'r', 0, NULL,
+      "Write the X server configuration to the configuration file, and exit, "
+      "without starting the graphical user interface." },
+
+    { "verbose", 'V', NVGETOPT_HAS_ARGUMENT, NULL,
+      "Controls how much information is printed.  Valid values are 'errors' "
+      "(print error messages), 'warnings' (print error and warning messages), "
+      "and 'all' (print error, warning and other informational messages).  By "
+      "default, only errors are printed." },
+
     { "assign", 'a', NVGETOPT_HAS_ARGUMENT, print_assign_help, NULL },
 
     { "query", 'q', NVGETOPT_HAS_ARGUMENT, print_query_help, NULL },
@@ -145,15 +168,41 @@ static void print_assign_help(void)
            "implied following the same rule as the --ctrl-display option.  "
            "If the X screen is not specified, then the assignment is made to "
            "all X screens.  Note that the '/' is only required when {DISPLAY} "
-           "is present.  The [{display devices}] portion is also optional; "
+           "is present.");
+
+    nv_msg(NULL, "");
+
+    nv_msg(BIGTAB, "{DISPLAY} can additionally include a target "
+           "specification to direct an assignment to something other than "
+           "an X screen.  A target specification is contained within brackets "
+           "and consists of a target type name, a colon, and the "
+           "target id.  The target type name can be one of \"screen\", "
+           "\"gpu\", or \"framelock\"; the target id is the index into the "
+           "list of targets (for that target type).  The target specification "
+           "can be used in {DISPLAY} wherever an X screen can be used, "
+           "following the syntax {host}:{display}[{target_type}:"
+           "{target_id}].  See the output of `nvidia-settings -q all` for "
+           "information on which target types can be used with which "
+           "attributes.  See the output of `nvidia-settings -q screens "
+           "-q gpus -q framelocks` for lists of targets for each "
+           "target type.");
+    
+    nv_msg(NULL, "");
+
+    nv_msg(BIGTAB, "The [{display devices}] portion is also optional; "
            "if it is not specified, then the attribute is assigned to all "
-           "display devices.  Some examples:");
+           "display devices.");
+
+    nv_msg(NULL, "");
+ 
+    nv_msg(BIGTAB, "Some examples:");
     
     nv_msg(NULL, "");
     
     nv_msg(BIGTAB TAB, "-a FSAA=5");
     nv_msg(BIGTAB TAB, "-a localhost:0.0/DigitalVibrance[CRT-0]=0");
     nv_msg(BIGTAB TAB, "--assign=\"SyncToVBlank=1\"");
+    nv_msg(BIGTAB TAB, "-a [gpu:0]/DigitalVibrance[DFP-1]=63");
     
 } /* print_assign_help() */
 
@@ -177,7 +226,10 @@ static void print_query_help(void)
     nv_msg(BIGTAB, "This queries the current value of the attribute "
            "{attribute name} on the X Display {DISPLAY}.  The format is "
            "the same as that for the '--assign' option, without "
-           "'={value}'.  Specify '-q all' to query all attributes.");
+           "'={value}'.  Specify '-q screens', '-q gpus', or '-q framelocks' "
+           "to query a list of X screens, GPUs, or Frame Lock devices, "
+           "respectively, that are present on the X Display {DISPLAY}.  "
+           "Specify '-q all' to query all attributes.");
 
 } /* print_query_help() */
 
@@ -263,8 +315,25 @@ Options *parse_command_line(int argc, char *argv[], char *dpy)
         switch (c) {
         case 'v': print_version(); exit(0); break;
         case 'h': print_help(); exit(0); break;
-        case 'l': op->load = 1; break;
+        case 'l': op->only_load = 1; break;
+        case 'n': op->no_load = 1; break;
+        case 'r': op->rewrite = 1; break;
         case 'c': op->ctrl_display = strval; break;
+        case 'V':
+            __verbosity = VERBOSITY_DEFAULT;
+            if (nv_strcasecmp(strval, "errors") == NV_TRUE) {
+                __verbosity = VERBOSITY_ERROR;
+            } else if (nv_strcasecmp(strval, "warnings") == NV_TRUE) {
+                __verbosity = VERBOSITY_WARNING;
+            } else if (nv_strcasecmp(strval, "all") == NV_TRUE) {
+                __verbosity = VERBOSITY_ALL;
+            } else {
+                nv_error_msg("Invalid verbosity level '%s'.  Please run "
+                             "`%s --help` for usage information.\n",
+                             strval, argv[0]);
+                exit(0);
+            }
+            break;
         case 'a':
             n = op->num_assignments;
             op->assignments = realloc(op->assignments, sizeof(char *) * (n+1));
