@@ -741,8 +741,8 @@ Bool display_add_modelines_from_server(nvDisplayPtr display, gchar **err_str)
     char *modeline_strs = NULL;
     char *str;
     int len;
-    ReturnStatus ret, ret1;
-    int major = 0, minor = 0;
+    ReturnStatus ret;
+
 
     /* Free any old mode lines */
     display_remove_modelines(display);
@@ -783,27 +783,7 @@ Bool display_add_modelines_from_server(nvDisplayPtr display, gchar **err_str)
             nv_error_msg(*err_str);
             goto fail;
         }
-
-        /*
-         * check the version of the NV-CONTROL protocol -- versions <=
-         * 1.13 had a bug in how they reported double scan modelines
-         * (vsyncstart, vsyncend, and vtotal were doubled); determine
-         * if this X server has this bug, so that we can use
-         * broken_doublescan_modelines to correctly compute the
-         * refresh rate.
-         */
-        modeline->broken_doublescan_modelines = 1;
-
-        ret = NvCtrlGetAttribute(display->gpu->handle,
-                                 NV_CTRL_ATTR_NV_MAJOR_VERSION, &major);
-        ret1 = NvCtrlGetAttribute(display->gpu->handle,
-                                  NV_CTRL_ATTR_NV_MINOR_VERSION, &minor);
-
-        if ((ret == NvCtrlSuccess) && (ret1 == NvCtrlSuccess) &&
-            ((major > 1) || ((major == 1) && (minor > 13)))) {
-            modeline->broken_doublescan_modelines = 0;
-        }
-
+        
         /* Add the modeline at the end of the display's modeline list */
         display->modelines = (nvModeLinePtr)xconfigAddListItem
             ((GenericListPtr)display->modelines, (GenericListPtr)modeline);
@@ -943,10 +923,6 @@ void screen_remove_display(nvDisplayPtr display)
         screen->displays_mask &= ~(display->device_mask);
         screen->num_displays--;
 
-        if (screen->primaryDisplay == display) {
-            screen->primaryDisplay = NULL;
-        }
-
         /* Clean up old references to the screen in the display */
         display_remove_modes(display);
         display->screen = NULL;
@@ -1737,7 +1713,7 @@ static int gpu_add_screen_from_server(nvGpuPtr gpu, int screen_id,
     nvScreenPtr screen;
     int val, tmp;
     ReturnStatus ret;
-    gchar *primary_str = NULL;
+
 
     /* Create the screen structure */
     screen = (nvScreenPtr)calloc(1, sizeof(nvScreen));
@@ -1771,7 +1747,7 @@ static int gpu_add_screen_from_server(nvGpuPtr gpu, int screen_id,
         *err_str = g_strdup_printf("Dynamic TwinView is disabled on "
                                    "screen %d.",
                                    screen_id);
-        nv_warning_msg(*err_str);
+        nv_error_msg(*err_str);
         goto fail;
     }
 
@@ -1805,31 +1781,8 @@ static int gpu_add_screen_from_server(nvGpuPtr gpu, int screen_id,
                        screen_id, NvCtrlGetTargetId(gpu->handle));
         goto fail;
     }
-    
-    /* Query & parse the screen's primary display */
-    screen->primaryDisplay = NULL;
-    ret = NvCtrlGetStringDisplayAttribute
-        (screen->handle,
-         0,
-         NV_CTRL_STRING_TWINVIEW_XINERAMA_INFO_ORDER,
-         &primary_str);
 
-    if (ret == NvCtrlSuccess) {
-        nvDisplayPtr d;
-        unsigned int  device_mask;
 
-        /* Parse the device mask */
-        parse_read_display_name(primary_str, &device_mask);
-
-        /* Find the matching primary display */
-        for (d = gpu->displays; d; d = d->next) {
-            if (d->screen == screen &&
-                d->device_mask & device_mask) {
-                screen->primaryDisplay = d;
-                break;
-            }
-        }
-    }
     /* Add the screen at the end of the gpu's screen list */
     gpu->screens =
         (nvScreenPtr)xconfigAddListItem((GenericListPtr)gpu->screens,
@@ -1882,12 +1835,10 @@ static Bool gpu_add_screens_from_server(nvGpuPtr gpu, gchar **err_str)
             nv_warning_msg("Failed to add screen %d to GPU-%d '%s'.",
                            pData[i], NvCtrlGetTargetId(gpu->handle),
                            gpu->name);
-            XFree(pData);
             goto fail;
         }
     }
 
-    XFree(pData);
     return TRUE;
 
 
@@ -2092,12 +2043,6 @@ static Bool layout_add_gpu_from_server(nvLayoutPtr layout, unsigned int gpu_id,
         goto fail;
     }
 
-    ret = NvCtrlGetAttribute(gpu->handle, NV_CTRL_DEPTH_30_ALLOWED,
-                             &(gpu->allow_depth_30));
-
-    if (ret != NvCtrlSuccess) {
-        gpu->allow_depth_30 = FALSE;
-    }
 
     /* Add the display devices to the GPU */
     if (!gpu_add_displays_from_server(gpu, err_str)) {
