@@ -66,17 +66,12 @@ typedef struct _CtkDisplayConfig
     GtkWidget *obj_layout;
 
     GtkWidget *chk_xinerama_enabled;
-    GtkWidget *chk_primary_display;
-
-    GtkWidget *notebook; /* Tabbed notebook for display and X screen pages */
 
 
     /* Display - Info */
-    GtkWidget *display_page;
+    GtkWidget *display_frame;
 
-    GtkWidget *mnu_display_model;
-    nvDisplayPtr *display_model_table; /* Lookup table for display modelname */
-    int display_model_table_len;
+    GtkWidget *txt_display_model;
     GtkWidget *txt_display_gpu;
 
     GtkWidget *btn_display_config;
@@ -107,7 +102,7 @@ typedef struct _CtkDisplayConfig
 
 
     /* X Screen - Info */
-    GtkWidget *screen_page;
+    GtkWidget *screen_frame;
 
     GtkWidget *txt_screen_num;
 
@@ -126,8 +121,8 @@ typedef struct _CtkDisplayConfig
     GtkWidget *btn_screen_metamode;
     GtkWidget *btn_screen_metamode_add;
     GtkWidget *btn_screen_metamode_delete;
-    
-    char *screen_depth_table;
+
+
     /* Dialogs */
     GtkWidget *dlg_display_config;
     GtkWidget *rad_display_config_disabled;
