@@ -29,11 +29,14 @@
 #include "parse.h"
 #include "config-file.h"
 
-int ctk_init_check(int *argc, char **argv[]);
+void ctk_init(int *argc, char **argv[]);
 
 char *ctk_get_display(void);
 
-void ctk_main(NvCtrlAttributeHandle **, int, ParsedAttribute*,
+void ctk_main(NvCtrlAttributeHandle **, int,
+              NvCtrlAttributeHandle **, int,
+              NvCtrlAttributeHandle **, int,
+              ParsedAttribute*,
               ConfigProperties*);
 
 #endif /* __CTK_UI_H__ */
