@@ -37,6 +37,8 @@
 #include "msg.h"
 #include "query-assign.h"
 
+extern int __verbosity;
+
 /* local prototypes */
 
 static int process_attribute_queries(int, char**, const char *);
@@ -92,7 +94,7 @@ CtrlHandles *nv_alloc_ctrl_handles(const char *display)
     ReturnStatus status;
     CtrlHandles *h, *pQueryHandle = NULL;
     NvCtrlAttributeHandle *handle;
-    int target, i, j, val, d, len;
+    int target, i, j, val, d, c, len;
     char *tmp;
 
     /* allocate the CtrlHandles struct */
@@ -152,7 +154,7 @@ CtrlHandles *nv_alloc_ctrl_handles(const char *display)
             }
             
             if (status != NvCtrlSuccess) {
-                nv_error_msg("Unable to determine number of NVIDIA "
+                nv_warning_msg("Unable to determine number of NVIDIA "
                              "%ss on '%s'.",
                              targetTypeTable[j].name,
                              XDisplayName(h->display));
@@ -234,11 +236,23 @@ CtrlHandles *nv_alloc_ctrl_handles(const char *display)
                                  NvCtrlAttributesStrError(status));
                     d = 0;
                 }
+                
+                status = NvCtrlGetAttribute(handle,
+                                            NV_CTRL_CONNECTED_DISPLAYS, &c);
+        
+                if (status != NvCtrlSuccess) {
+                    nv_error_msg("Error querying connected displays on "
+                                 "%s %d (%s).", targetTypeTable[j].name, i,
+                                 NvCtrlAttributesStrError(status));
+                    c = 0;
+                }
             } else {
                 d = 0;
+                c = 0;
             }
              
             h->targets[target].t[i].d = d;
+            h->targets[target].t[i].c = c;
 
             /*
              * store this handle so that we can use it to query other
@@ -341,7 +355,8 @@ static int process_attribute_queries(int num, char **queries,
 
         /* special case the target type queries */
         
-        if (nv_strcasecmp(queries[query], "screens")) {
+        if (nv_strcasecmp(queries[query], "screens") ||
+            nv_strcasecmp(queries[query], "xscreens")) {
             query_all_targets(display_name, X_SCREEN_TARGET);
             continue;
         }
@@ -681,8 +696,8 @@ static void print_valid_values(char *name, uint32 flags,
         }
 
         if (flags & NV_PARSER_TYPE_PACKED_ATTRIBUTE) {
-            nv_msg(INDENT, "Valid values for '%s' are: [%s], [%s].", name, str2,
-                   str);
+            nv_msg(INDENT, "Valid values for '%s' are: [%s], [%s].", name, str,
+                   str2);
         } else {
             nv_msg(INDENT, "Valid values for '%s' are: %s.", name, str);
         }
@@ -823,35 +838,65 @@ static int query_all(const char *display_name)
 
                 if ((t->d & mask) == 0x0) continue;
                 
-                status = NvCtrlGetValidDisplayAttributeValues
-                    (t->h, mask, a->attr, &valid);
+                if (a->flags & NV_PARSER_TYPE_STRING_ATTRIBUTE) {
+                    char *tmp_str = NULL;
 
-                if (status == NvCtrlAttributeNotAvailable) goto exit_bit_loop;
+                    status = NvCtrlGetValidStringDisplayAttributeValues(t->h, mask,
+                                                                        a->attr, &valid);
+
+                    if (status == NvCtrlAttributeNotAvailable) goto exit_bit_loop;
+
+                    if (status != NvCtrlSuccess) {
+                        nv_error_msg("Error while querying valid values for "
+                                     "attribute '%s' on %s (%s).",
+                                     a->name, t->name,
+                                     NvCtrlAttributesStrError(status));
+                        goto exit_bit_loop;
+                    }
+
+                    status = NvCtrlGetStringDisplayAttribute(t->h, mask, a->attr, &tmp_str);
+
+                    if (status == NvCtrlAttributeNotAvailable) goto exit_bit_loop;
+
+                    if (status != NvCtrlSuccess) {
+                        nv_error_msg("Error while querying attribute '%s' "
+                                     "on %s (%s).", a->name, t->name,
+                                     NvCtrlAttributesStrError(status));
+                        goto exit_bit_loop;
+                    }
+
+                    nv_msg("  ",  "Attribute '%s' (%s%s): %s ", a->name, t->name, "", tmp_str);
+                    free(tmp_str);
+                    tmp_str = NULL;
+
+                } else {
+                    status = NvCtrlGetValidDisplayAttributeValues(t->h, mask, a->attr, &valid);
+
+                    if (status == NvCtrlAttributeNotAvailable) goto exit_bit_loop;
                 
-                if (status != NvCtrlSuccess) {
-                    nv_error_msg("Error while querying valid values for "
-                                 "attribute '%s' on %s (%s).",
-                                 a->name, t->name,
-                                 NvCtrlAttributesStrError(status));
-                    goto exit_bit_loop;
+                    if (status != NvCtrlSuccess) {
+                        nv_error_msg("Error while querying valid values for "
+                                     "attribute '%s' on %s (%s).",
+                                     a->name, t->name,
+                                     NvCtrlAttributesStrError(status));
+                        goto exit_bit_loop;
+                    }
+
+                    status = NvCtrlGetDisplayAttribute(t->h, mask, a->attr, &val);
+
+                    if (status == NvCtrlAttributeNotAvailable) goto exit_bit_loop;
+
+                    if (status != NvCtrlSuccess) {
+                        nv_error_msg("Error while querying attribute '%s' "
+                                     "on %s (%s).", a->name, t->name,
+                                     NvCtrlAttributesStrError(status));
+                        goto exit_bit_loop;
+                    }
+
+                    print_queried_value(t, &valid, val, a->flags, a->name, mask, INDENT);
+
+                    print_valid_values(a->name, a->flags, valid);
                 }
-
-                status = NvCtrlGetDisplayAttribute(t->h, mask, a->attr, &val);
-
-                if (status == NvCtrlAttributeNotAvailable) goto exit_bit_loop;
-
-                if (status != NvCtrlSuccess) {
-                    nv_error_msg("Error while querying attribute '%s' "
-                                 "on %s (%s).",
-                                 a->name, t->name,
-                                 NvCtrlAttributesStrError(status));
-                    goto exit_bit_loop;
-                }
-
-                print_queried_value(t, &valid, val, a->flags,
-                                    a->name, mask, INDENT);
-                
-                print_valid_values(a->name, a->flags, valid);
                 
                 nv_msg(NULL,"");
                 
@@ -881,6 +926,192 @@ static int query_all(const char *display_name)
 
 
 
+/*
+ * print_target_display_connections() - Print a list of all the
+ * display devices connected to the given target (GPU/X Screen)
+ */
+
+static int print_target_display_connections(CtrlHandleTarget *t)
+{
+    unsigned int bit;
+    char *tmp_d_str, *name;
+    ReturnStatus status;
+    int plural;
+
+    if (t->c == 0) {
+        nv_msg("      ", "Is not connected to any display devices.");
+        nv_msg(NULL, "");
+        return NV_TRUE;
+    }
+
+    plural = (t->c & (t->c - 1)); /* Is more than one bit set? */
+
+    nv_msg("      ", "Is connected to the following display device%s:",
+           plural ? "s" : "");
+
+    for (bit = 1; bit; bit <<= 1) {
+        if (!(bit & t->c)) continue;
+        
+        status =
+            NvCtrlGetStringDisplayAttribute(t->h, bit,
+                                            NV_CTRL_STRING_DISPLAY_DEVICE_NAME,
+                                            &name);
+        if (status != NvCtrlSuccess) name = strdup("Unknown");
+        
+        tmp_d_str = display_device_mask_to_display_device_name(bit);
+        nv_msg("        ", "%s (%s: 0x%0.8X)", name, tmp_d_str, bit);
+        
+        free(name);
+        free(tmp_d_str);
+    }
+    nv_msg(NULL, "");
+
+    return NV_TRUE;
+
+} /* print_target_display_connections() */
+
+
+
+/*
+ * get_vcsc_name() Returns the VCSC product name of the given
+ * VCSC target.
+ */
+
+static char * get_vcsc_name(NvCtrlAttributeHandle *h)
+{
+    char *product_name;
+    ReturnStatus status;
+
+    status = NvCtrlGetStringAttribute(h, NV_CTRL_STRING_VCSC_PRODUCT_NAME,
+                                      &product_name);
+    
+    if (status != NvCtrlSuccess) return strdup("Unknown");
+    
+    return product_name;
+
+} /* get_vcsc_name() */
+
+
+
+/*
+ * get_gpu_name() Returns the GPU product name of the given
+ * GPU target.
+ */
+
+static char * get_gpu_name(NvCtrlAttributeHandle *h)
+{
+    char *product_name;
+    ReturnStatus status;
+
+    status = NvCtrlGetStringAttribute(h, NV_CTRL_STRING_PRODUCT_NAME,
+                                      &product_name);
+    
+    if (status != NvCtrlSuccess) return strdup("Unknown");
+    
+    return product_name;
+
+} /* get_gpu_name() */
+
+
+
+/*
+ * print_target_connections() Prints a list of all targets connected
+ * to a given target using print_func if the connected targets require
+ * special handling.
+ */
+
+static int print_target_connections(CtrlHandles *h,
+                                    CtrlHandleTarget *t,
+                                    unsigned int attrib,
+                                    unsigned int target_index)
+{
+    int *pData;
+    int len, i;
+    ReturnStatus status;
+    char *product_name;
+    char *target_name;
+
+
+    /* Query the connected targets */
+
+    status =
+        NvCtrlGetBinaryAttribute(t->h, 0, attrib,
+                                 (unsigned char **) &pData,
+                                 &len);
+    if (status != NvCtrlSuccess) return NV_FALSE;
+
+    if (pData[0] == 0) {
+        nv_msg("      ", "Is not connected to any %s.",
+               targetTypeTable[target_index].name);
+        nv_msg(NULL, "");
+
+        XFree(pData);
+        return NV_TRUE;
+    }
+
+    nv_msg("      ", "Is connected to the following %s%s:",
+           targetTypeTable[target_index].name,
+           ((pData[0] > 1) ? "s" : ""));
+
+
+    /* List the connected targets */
+
+    for (i = 1; i <= pData[0]; i++) {
+        
+        if (pData[i] >= 0 &&
+            pData[i] < h->targets[target_index].n) {
+
+            target_name = h->targets[target_index].t[ pData[i] ].name;
+
+            switch (target_index) {
+            case GPU_TARGET:
+                product_name =
+                    get_gpu_name(h->targets[target_index].t[ pData[i] ].h);
+                break;
+                
+            case VCSC_TARGET:
+                product_name =
+                    get_vcsc_name(h->targets[target_index].t[ pData[i] ].h);
+                break;
+                
+            case FRAMELOCK_TARGET:
+            case X_SCREEN_TARGET:
+            default:
+                product_name = NULL;
+                break;
+            }
+
+        } else {
+            target_name = NULL;
+            product_name = NULL;
+        }
+
+        if (!target_name) {
+            nv_msg("        ", "[?] Unknown %s",
+                   targetTypeTable[target_index].name);
+
+        } else if (product_name) {
+            nv_msg("        ", "[%d] %s (%s)",
+                   pData[i], target_name, product_name);
+
+        } else {
+            nv_msg("        ", "[%d] %s (%s %d)",
+                   pData[i], target_name,
+                   targetTypeTable[target_index].name,
+                   pData[i]);
+        }
+
+        free(product_name);
+    }
+    nv_msg(NULL, "");
+
+    XFree(pData);
+    return NV_TRUE;
+
+} /* print_target_connections() */
+
+
+
 /*
  * query_all_targets() - print a list of all the targets (of the
  * specified type) accessible via the Display connection.
@@ -987,6 +1218,48 @@ static int query_all_targets(const char *display_name, const int target_index)
         nv_msg(NULL, "");
         
         free(product_name);
+
+        /* Print connectivity information */
+        if (__verbosity >= VERBOSITY_ALL) {
+            if (targetTypeTable[table_index].uses_display_devices) {
+                print_target_display_connections(t);
+            }
+
+            switch (target_index) {
+            case GPU_TARGET:
+                print_target_connections
+                    (h, t, NV_CTRL_BINARY_DATA_XSCREENS_USING_GPU,
+                     X_SCREEN_TARGET);
+                print_target_connections
+                    (h, t, NV_CTRL_BINARY_DATA_FRAMELOCKS_USED_BY_GPU,
+                     FRAMELOCK_TARGET);
+                print_target_connections
+                    (h, t, NV_CTRL_BINARY_DATA_VCSCS_USED_BY_GPU,
+                     VCSC_TARGET);
+                break;
+
+            case X_SCREEN_TARGET:
+                print_target_connections
+                    (h, t, NV_CTRL_BINARY_DATA_GPUS_USED_BY_XSCREEN,
+                     GPU_TARGET);
+                break;
+
+            case FRAMELOCK_TARGET:
+                print_target_connections
+                    (h, t, NV_CTRL_BINARY_DATA_GPUS_USING_FRAMELOCK,
+                     GPU_TARGET);
+                break;
+
+            case VCSC_TARGET:
+                print_target_connections
+                    (h, t, NV_CTRL_BINARY_DATA_GPUS_USING_VCSC,
+                     GPU_TARGET);
+                break;
+
+            default:
+                break;
+            }
+        }
     }
     
     nv_free_ctrl_handles(h);
@@ -1050,27 +1323,47 @@ static int process_parsed_attribute_internal(CtrlHandleTarget *t,
         }
 
     } else { /* query */
-        
-        status = NvCtrlGetDisplayAttribute(t->h, d, a->attr, &a->val);
+        if (a->flags & NV_PARSER_TYPE_STRING_ATTRIBUTE) {
+            char *tmp_str = NULL;
+            status = NvCtrlGetStringDisplayAttribute(t->h, d, a->attr, &tmp_str);
 
-        if (status == NvCtrlAttributeNotAvailable) {
-            nv_warning_msg("Error querying attribute '%s' specified %s; "
-                           "'%s' is not available on %s%s.",
-                           a->name, whence, a->name,
-                           t->name, str);
-        } else if (status != NvCtrlSuccess) {
-            nv_error_msg("Error while querying attribute '%s' "
-                         "(%s%s) specified %s (%s).",
-                         a->name, t->name, str, whence,
-                         NvCtrlAttributesStrError(status));
-            return NV_FALSE;
-        } else {
+            if (status == NvCtrlAttributeNotAvailable) {
+                nv_warning_msg("Error querying attribute '%s' specified %s; "
+                               "'%s' is not available on %s%s.",
+                               a->name, whence, a->name,
+                               t->name, str);
+            } else if (status != NvCtrlSuccess) {
+                nv_error_msg("Error while querying attribute '%s' "
+                             "(%s%s) specified %s (%s).",
+                             a->name, t->name, str, whence,
+                             NvCtrlAttributesStrError(status));
+                return NV_FALSE;
+            } else {
+                nv_msg("  ",  "Attribute '%s' (%s%s): %s", a->name, t->name, str, tmp_str);
+                free(tmp_str);  
+                tmp_str = NULL;
+            }
+        } else { 
+            status = NvCtrlGetDisplayAttribute(t->h, d, a->attr, &a->val);
 
-            print_queried_value(t, &valid, a->val, a->flags, a->name, d, "  ");
-
-            print_valid_values(a->name, a->flags, valid);
+            if (status == NvCtrlAttributeNotAvailable) {
+                nv_warning_msg("Error querying attribute '%s' specified %s; "
+                               "'%s' is not available on %s%s.",
+                               a->name, whence, a->name,
+                               t->name, str);
+            } else if (status != NvCtrlSuccess) {
+                nv_error_msg("Error while querying attribute '%s' "
+                             "(%s%s) specified %s (%s).",
+                             a->name, t->name, str, whence,
+                             NvCtrlAttributesStrError(status));
+                return NV_FALSE;
+            } else {
+                print_queried_value(t, &valid, a->val, a->flags, a->name, d, "  ");
+                
+                print_valid_values(a->name, a->flags, valid);
+            }
         }
-    }
+    } /* query */
 
     return NV_TRUE;
 
@@ -1211,7 +1504,7 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
          */
 
         if (!h->targets[target].t[a->target_id].h) {
-            nv_error_msg("Invalid %s %d specified %s (NV-CONTROL extension "
+            nv_warning_msg("Invalid %s %d specified %s (NV-CONTROL extension "
                          "not supported on %s %d).",
                          target_type_name,
                          a->target_id, whence,
@@ -1280,10 +1573,18 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
                     (a->display_device_mask);
                 tmp_d_str1 = display_device_mask_to_display_device_name(t->d);
 
-                nv_error_msg("Invalid display device %s specified "
-                             "%s (the currently enabled display devices "
-                             "are %s on %s).",
-                             tmp_d_str0, whence, tmp_d_str1, t->name);
+                if (tmp_d_str1 && (*tmp_d_str1 != '\0')) {
+                    nv_error_msg("Invalid display device %s specified "
+                                 "%s (the currently enabled display devices "
+                                 "are %s on %s).",
+                                 tmp_d_str0, whence, tmp_d_str1, t->name);
+                } else {
+                    nv_error_msg("Invalid display device %s specified "
+                                 "%s (there are currently no enabled display "
+                                 "devices on %s).",
+                                 tmp_d_str0, whence, t->name);
+                }
+
                 free(tmp_d_str0);
                 free(tmp_d_str1);
                 
@@ -1325,6 +1626,37 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
             continue;
         }
 
+        /*
+         * If we are assigning, and the value for this attribute is a
+         * display device, then we need to validate the value against
+         * the mask of enabled display devices.
+         */
+        
+        if (assign && (a->flags & NV_PARSER_TYPE_VALUE_IS_DISPLAY)) {
+            if (a->flags & NV_PARSER_TYPE_ASSIGN_ALL_DISPLAYS) {
+                a->val = t->d;
+            }
+            
+            if ((t->d & a->val) != a->val) {
+
+                tmp_d_str0 =
+                    display_device_mask_to_display_device_name(a->val);
+
+                tmp_d_str1 = display_device_mask_to_display_device_name(t->d);
+
+                nv_error_msg("The attribute '%s' specified %s cannot be "
+                             "assigned the value of %s (the currently enabled "
+                             "display devices are %s on %s).",
+                             a->name, whence, tmp_d_str0, tmp_d_str1,
+                             t->name);
+                
+                free(tmp_d_str0);
+                free(tmp_d_str1);
+                
+                continue;
+            }
+        }
+
         /*
          * If we are assigning, and the value for this attribute is
          * not allowed to be zero, check that the value is not zero.
@@ -1350,41 +1682,14 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
             }
         }
         
-        /*
-         * If we are assigning, and the value for this attribute is a
-         * display device, then we need to validate the value against
-         * the mask of enabled display devices.
-         */
-        
-        if (assign && (a->flags & NV_PARSER_TYPE_VALUE_IS_DISPLAY)) {
-            
-            if ((t->d & a->val) != a->val) {
-
-                tmp_d_str0 =
-                    display_device_mask_to_display_device_name(a->val);
-
-                tmp_d_str1 = display_device_mask_to_display_device_name(t->d);
-
-                nv_error_msg("The attribute '%s' specified %s cannot be "
-                             "assigned the value of %s (the currently enabled "
-                             "display devices are %s on %s).",
-                             a->name, whence, tmp_d_str0, tmp_d_str1,
-                             t->name);
-                
-                free(tmp_d_str0);
-                free(tmp_d_str1);
-                
-                continue;
-            }
-        }
-
         /*
          * If we are dealing with a frame lock attribute on a non-frame lock
          * target type, make sure frame lock is available.
          *
          * Also, when setting frame lock attributes on non-frame lock targets,
          * make sure frame lock is disabled.  (Of course, don't check this for
-         * the "enable frame lock" attribute.)
+         * the "enable frame lock" attribute, and special case the "Test
+         * Signal" attribute.)
          */
 
         if ((a->flags & NV_PARSER_TYPE_FRAMELOCK) &&
@@ -1409,7 +1714,7 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
                 continue;
             }
 
-            /* Don't assign if frame lock is enabled */
+            /* Do assignments based on the frame lock sync status */
 
             if (assign && (a->attr != NV_CTRL_FRAMELOCK_SYNC)) {
                 int enabled;
@@ -1424,12 +1729,20 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
                                  NvCtrlAttributesStrError(status));
                     continue;
                 }
-                
-                if (enabled != NV_CTRL_FRAMELOCK_SYNC_DISABLE) {
-                    nv_error_msg("The attribute '%s' specified %s cannot be "
-                                 "assigned;  frame lock sync is currently "
-                                 "enabled on %s.",
-                                 a->name, whence, t->name);
+
+                if (a->attr == NV_CTRL_FRAMELOCK_TEST_SIGNAL) {
+                    if (enabled != NV_CTRL_FRAMELOCK_SYNC_ENABLE) {
+                        nv_error_msg("The attribute '%s' specified %s cannot "
+                                     "be assigned;  frame lock sync is "
+                                     "currently disabled on %s.",
+                                     a->name, whence, t->name);
+                        continue;
+                    }
+                } else if (enabled != NV_CTRL_FRAMELOCK_SYNC_DISABLE) {
+                    nv_warning_msg("The attribute '%s' specified %s cannot be "
+                                   "assigned;  frame lock sync is currently "
+                                   "enabled on %s.",
+                                   a->name, whence, t->name);
                     continue;
                 }
             }
@@ -1442,12 +1755,17 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
             mask = (1 << bit);
 
             if (((mask & display_devices) == 0x0) &&
-                (targetTypeTable[target_type_index].uses_display_devices)) {
+                (targetTypeTable[target_type_index].uses_display_devices) &&
+                (t->d)) {
                 continue;
             }
-            
-            status = NvCtrlGetValidDisplayAttributeValues(t->h, mask,
-                                                          a->attr, &valid);
+
+            if (a->flags & NV_PARSER_TYPE_STRING_ATTRIBUTE) {
+                status = NvCtrlGetValidStringDisplayAttributeValues(t->h, mask, a->attr, &valid);
+            } else {
+                status = NvCtrlGetValidDisplayAttributeValues(t->h, mask, a->attr, &valid);
+            }
+
             if (status != NvCtrlSuccess) {
                 if (status == NvCtrlAttributeNotAvailable) {
                     nv_warning_msg("Attribute '%s' specified %s is not "
@@ -1480,12 +1798,14 @@ int nv_process_parsed_attribute(ParsedAttribute *a, CtrlHandles *h,
             
             /*
              * if this attribute is not per-display device, or this
-             * target does not know about display devices, then once
-             * through this loop is enough.
+             * target does not know about display devices, or this target
+             * does not have display devices, then once through this loop
+             * is enough.
              */
             
             if ((!(valid.permissions & ATTRIBUTE_TYPE_DISPLAY)) ||
-                (!(targetTypeTable[target_type_index].uses_display_devices))) {
+                (!(targetTypeTable[target_type_index].uses_display_devices)) ||
+                (!(t->d))) {
                 break;
             }
 
